Package: ewens
Title: Ewens Distribution
Version: 0.1.0
Authors@R: 
    person("Chris", "Hanretty", , "chris.hanretty@gmail.com", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-8932-9405"))
Description: Implements the probability mass function of, and random draws from, the Ewens distribution, a probability distribution over partitions of integer, as described in Ewens (1972) <doi:10.1016/0040-5809(72)90035-4>.
License: MIT + file LICENSE
Encoding: UTF-8
URL: https://github.com/chrishanretty/ewens
BugReports: https://github.com/chrishanretty/ewens/issues
RoxygenNote: 7.3.2
Imports: copula (>= 1.0)
Suggests: knitr, quarto
VignetteBuilder: quarto
NeedsCompilation: yes
Packaged: 2026-05-14 11:01:47 UTC; chanret
Author: Chris Hanretty [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-8932-9405>)
Maintainer: Chris Hanretty <chris.hanretty@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-19 09:30:14 UTC
Built: R 4.5.2; x86_64-apple-darwin20; 2026-05-19 09:43:45 UTC; unix
Archs: ewens.so.dSYM

# Generated by roxygen2: do not edit by hand

export(dewens)
export(dewens_k)
export(ewens_mle)
export(gcrp)
export(rewens)
export(rgem)
importFrom(stats,rbeta)
importFrom(stats,uniroot)
useDynLib(ewens, .registration = TRUE)

# ewens

<!-- badges: start -->
<!-- badges: end -->

The `R` package `ewens` gives functions for the [Ewens sampling distribution](https://en.wikipedia.org/wiki/Ewens%27s_sampling_formula). The Ewens distribution is a probability distribution over partitions of an integer $n$. It is governed by a single non-negative parameter, $\theta$. The larger the value of $\theta$, the more parts there are in the partition. If $\theta$ is equal to zero, there is just a single partition with size $n$. 

The Ewens distribution can be thought of as a distribution over partitions of an integer, but it originated in population genetics to give the probability of alleles in a sample, and sampling from the Ewens distribution involves running through the [Chinese Restaurant Process](https://en.wikipedia.org/wiki/Chinese_restaurant_process) $n$ times. For further information on the Ewens sampling formula, see Simon Tavaré's [The magical Ewens sampling formula](https://www.columbia.edu/cu/simontavare/STpapers-pdf/t20b.pdf) or Harry Crane's [The Ubiquitous Ewens Sampling Formula](https://www.jstor.org/stable/24780825).

## Installation

You can install the development version of `ewens` like so:

``` r
if (!requireNamespace("remotes", quietly = TRUE)) {
  install.packages("remotes")
}
remotes::install_github("chrishanretty/ewens")
```

## Drawing from the Ewens distribution

Suppose I wish to divide a class of 24 into groups of different sizes. I set $\theta$ to one, and call `rewens`

``` r
set.seed(2923)
library(ewens)

allocation <- rewens(24, theta = 1)
```

The vector `allocation` gives the groups to which each student has been assigned. The sizes of the groups can be recovered by tabulating the result.

``` r
table(allocation)
```

## The probability mass function of the Ewens distribution

Suppose a student challenges their allocation into a particular group. What can we say about the probability of the particular allocation, given $\theta = 1$?

``` r
pr <- dewens(allocation, theta = 1)
pr
```

The probability of drawing this particular allocation is very low --
but then again, given that there are 1575 different partitions of
twenty-four, we should not be too surprised.

## The number of parts in the partition

It is also possible to calculate the probability that there are $x$
parts in a partition of $n$ given $\theta$. This is done using
`dewens_k`. Thus, the probability of creating the groups above is
given by

```{r}
dewens_k(k = length(unique(allocation)),
         n = 24,
         theta = 1)
```

#' Calculate expected number of classes in a sample of size n given theta
#'
#' @description
#' The expected number of classes from the Ewens distribution is given by \eqn{\theta \sum_{j=1}^{n} \frac{1}{\theta + j - 1}}. This is often more convenient than integrating across the PMF given by `dewens_k`
#' 
#' @param n The sample size
#' @param theta The non-negative parameter governing expected sample diversity
#' 
ewens_k_exact <- function(n, theta) {
    theta * sum(1 / (theta + seq(0, n - 1)))
}

#' Maximum likelihood estimate of theta given sample vector with class memberships
#'
#' @param x A vector containing class memberships; sample size n and number of classes k are calculated from this
#' 
#' @importFrom stats uniroot
#' @returns A scalar giving the estimate of theta
#' @export
ewens_mle <- function(x) {
    n <- length(x)
    K <- length(unique(x))
    score <- function(theta) K/theta - sum(1/(theta + seq(0, n-1)))
    if (n == 1) {
        ### We can't learn from this
        return(NaN)
    }
    stats::uniroot(score, interval = c(1e-6, n))$root
}

#' Draw from the Griffiths-Engen-McCloskey distribution
#'
#' @details
#' The Griffiths-Engen-McCloskey distribution is the infinite dimensional counterpart to the Ewens sampling distribution. This function does not return an infinite dimensional vector(!), but returns a vector of shares creating by a "stick-breaking" construction. The vector of shares is returned after `trunc_at` sticks are broken; this can mean that there is still a non-negligible residual amount. 
#' 
#' @param alpha A parameter between zero and one
#' @param theta A parameter which must be greater than -alpha
#' @param trunc_at An integer which specifies the maximum number of components to return
#' @returns A vector of shares of length `trunc_at` which may sum to less than one
#'
#' @importFrom stats rbeta
#' @export
rgem <- function(alpha = 0, theta = 1, trunc_at = 500) {
  if (alpha < 0 || alpha > 1) stop("alpha must be in [0,1]")
  if (theta < -alpha) stop("theta must be >= -alpha")
  
  k <- seq_len(trunc_at)
  v <- stats::rbeta(trunc_at, 1 - alpha, theta + k * alpha)
  
  remaining <- cumprod(c(1, 1 - v[-trunc_at]))
  s <- remaining * v
  
  s
}

#' Probability mass function for the number of classes from a Ewens distribution
#'
#' @details
#' The number of classes from a Ewens distribution with parameter \eqn{\theta} is given by the expression \deqn{Pr(K = k) = \lvert{} S^k_n \rvert{} \frac{\theta^k}{\theta (\theta + 1) ... (\theta + n - 1)}}, where \eqn{\lvert{}S^k_n \rvert{}} is the absolute value of a Stirling number of the first kind.
#' 
#' @param k An integer number of classes at which to evaluate the PMF
#' @param n A sample size not less than k
#' @param theta A non-negative parameter governing the expected sample diversity.
#' @returns The probability of observing k classes
#' @examples
#' x <- sample(LETTERS, 120, replace = TRUE)
#' dewens_k(1, 20, theta = 1) ## Pretty unlikely we just see one class
#'
#' @export
dewens_k <- function(k, n, theta) {
    abs(copula::Stirling1(n, k)) * (theta ^ k) /
        exp(sum(log(theta + seq_len(n) - 1)))
}

#' Draw from the Ewens distribution
#'
#' Returns a vector with class membership
#'
#' @details
#' Although this command is described as sampling from the Ewens distribution, it is easier to think of it as a particular instantiation of the Chinese Restaurant Process, run for n "customers". The $j$th customer
#'  - sits at a new table with probability \eqn{\frac{\theta}{j - 1 + \theta}}, or
#'  - sits at an occupied table with probability \eqn{\frac{c}{j - 1 + \theta}}
#' where $c$ is the number of customers already at each table.
#' 
#' @param n The sample size.
#' @param theta A non-negative parameter governing the expected sample diversity.
#' @returns A vector of length n consisting of numeric class labels.
#' @examples
#' rewens(100, 1)
#' rewens(120, 0.5)
#' rewens(10, 0) ## equal to rep(1, 10)
#' @export
rewens <- function(n, theta = 1) {
    .Call("rewens_c", as.integer(n), as.numeric(theta), PACKAGE = "ewens")
}

#' Probability mass function for the Ewens distribution
#'
#' @description
#' Gives the probability mass function for the Ewens distribution, as described in Ewens, Warren (1972). "The sampling theory of selectively neutral alleles". *Theoretical Population Biology*. 3: 87–112. doi:10.1016/0040-5809(72)90035-4.
#'
#' @details
#' The probability of a vector of counts \eqn{m_1, ..., m_n} is given by the expression \deqn{\frac{n!}{\theta (\theta + 1) ... (\theta + n - 1)}\prod_{j=1}^n \frac{\theta^{m_j}}{j^{m_j} m_j!}}
#' 
#' @param x A vector giving class memberships of each observation in the sample
#' @param theta A non-negative parameter governing the expected sample diversity.
#' @param log if TRUE, probabilities are given as log(p). Default is FALSE.
#' @returns A numeric vector giving a probability (or if `log = TRUE`, a log probability)
#' @examples
#' x <- sample(LETTERS, 120, replace = TRUE)
#' dewens(x, theta = 1)
#' dewens(x, theta = 0) ## returns NaN since vector incompatible with zero diversity
#'
#' @export
dewens <- function(x, theta = 1, log = FALSE) {
    if (theta < 0) {
        stop("Parameter theta must be non-negative. ")
    }
    ### Get counts
    tab <- tabulate(factor(x))
    mvec   <- tabulate(tab, nbins = max(tab))
    n      <- length(x)
    j_seq  <- seq_along(mvec)

    log_pmf <- lfactorial(n) -
        sum(log(theta + seq_len(n) - 1)) +   # log rising factorial
        sum(mvec * log(theta)                  # theta^{a_j}
            - mvec * log(j_seq)               # j^{a_j}
            - lfactorial(mvec))               # a_j!

    if (log) {
        return(log_pmf)
    } else {
        return(exp(log_pmf))
    }
}

#' Draw from a generalized Chinese Restaurant Process
#' 
#' @param n The sample size.
#' @param theta A non-negative parameter governing the expected sample diversity.
#' @param alpha A parameter between zero and one inclusive governing the expected sample diversity
#' @returns A vector of length n consisting of numeric class labels.
#' @examples
#' rewens(100, 1)
#' rewens(120, 0.5)
#' rewens(10, 0)
#' @export
gcrp <- function(n, alpha = 0, theta = 1) {
    .Call("gcrp_c", as.integer(n), as.numeric(alpha),
          as.numeric(theta), PACKAGE = "ewens")
}

---
title: "Sampling from the Ewens distribution"
vignette: >
  %\VignetteIndexEntry{Sampling from the Ewens distribution}
  %\VignetteEngine{quarto::html}
  %\VignetteEncoding{UTF-8}
knitr:
  opts_chunk:
    collapse: true
    comment: '#>'
---

```{r}
#| label: setup
library(ewens)
```

# The Ewens distribution

The Ewens distribution is a distribution over partitions of
integers. It has a single non-negative parameter $\theta$, which
determines how many parts one can expect in the partition: as $\theta$
goes to zero, the Ewens distribution returns a single part; as
$\theta$ goes to infinity, the Ewens distribution returns $n$ parts
each with a value of one.

The Ewens distribution also gives the expected count of objects
belonging to category $k$ when a sample of size $n$ is drawn from an
infinitely large population. For this reason, the formula for the
probability mass function of the Ewens distribution is also called the
Ewens sampling formula [@ewens1972sampling]. 

Suppose that we have $m_1$ categories which appear once in the sample,
$m_2$ categories which appear twice in the sample, and so on. Then the
probability of the vector $m$ is as follows:

$$
p(m_1, ..., m_n; \theta) = \frac{n!}{\theta (\theta + 1) ... (\theta + n - 1)} \prod_{j=1}^n \frac{\theta^{m_j}}{j^{m_j} m_j! }
$$ {#eq-ewens-pmf}

The Ewens distribution is related to other constructions in
statistics. For example: to *sample* from a Ewens distribution, and
return a sample of size $n$ where each sample observation belongs to
some category $k$, we follow the Chinese Restaurant Process
[@aldous1983exchangeability, p. 92] for $n$ steps. We imagine $n$
customers entering the restaurant and choosing to sit at one of an
infinite number of infinitely large banquet tables. The first customer
enters, and sits at a table. The second customer enters, and either

 - sits at a new table with probability $\frac{\theta}{2 - 1 + \theta}$, or
 - sits at the occupied table with probability $\frac{1}{2 - 1 + \theta}$

The third customer enters, and either

 - sits at a new table with probability $\frac{\theta}{3 - 1 + \theta}$, or 
 - sits at one of the occupied tables with probability $\frac{c}{3 -
   1 + \theta}$, where $c$ is the count of people at that table.

In general, the $j$th customer

 - sits at a new table with probability $\frac{\theta}{j - 1 + \theta}$, or 
 - sits at an occupied table with probability $\frac{c}{j - 1 + \theta}$
 
From the Ewens sampling formula, we can derive an expression for the
probability of there being $k$ parts in the partition of $n$ -- or
alternately, the probability distribution of the number of tables in
our Chinese restaurant. This formula is sometimes referred to as the
Chinese restaurant table *distribution* -- or at least so says
[Wikipedia](https://en.wikipedia.org/wiki/Chinese_restaurant_process#Expected_number_of_tables). This
probability mass function is given by

$$
Pr(K = k) = \lvert{} S^k_n \rvert{} \frac{\theta^k}{\theta (\theta + 1) ... (\theta + n - 1)}
$$ {#eq-crt-pmf}

where $\lvert{}S^k_n$\rvert{}$ is the absolute value of a Stirling
number of the first kind. If we are solely interested in the expected
number of tables, rather than the entire distribution, we can use the
following expression:

$$
\newcommand{\Expect}{\operatorname{\mathbb{E}}}
\Expect{[k]} = \theta \sum_{j=1}^{n} \frac{1}{\theta + j - 1}
$$ {#eq-crt-expectation}

Thus, if we know our sample size and the value of $\theta$, we can
work out the expected number of classes. One might think that it
wouldn't be possible to apply this logic in reverse, and arrive at an
expression for $\theta$ in terms of the sample size and the observed
number of classes. After all, the observed number of classes reports
much less information than the full vector of frequencies $m_1, ...,
m_n$. This seems plausible on the face of it, and yet it is wrong. As
@tavare2021magical writes, discussing the Ewens sampling formula in
its original context of population genetics:

> "In statistical parlance, the number $K_n$ of different alleles
> observed in the sample is sufficient for the parameter $\theta$. It
> follows from the Rao–Blackwell theorem that estimation of \theta
> should be based on $K_n$; earlier, estimation of \theta had been
> based on the observed allele frequencies"

The log-likelihood for $\theta$ given $K$ and $n$ is:

$$
K \log \theta - \sum_{i=0}^{n-1}\log (\theta + i) + constant
$$

If, for the purposes of maximum likelihood estimation, we
differentiate this and set to zero, we get:

$$
\frac{\mathrm{d} \ell}{\mathrm{d} \theta} = \frac{K}{\theta} - \sum_{i=0}^{n-1}\frac{1}{\theta +i}
$$ {#eq-mle}

Useful readings on the Ewens distribution include @crane2016ubiquitous
and @tavare2021magical; @ewens1972sampling is probably of historical
interest only.

# R implementation

The `dewens` function gives the probability mass function in
@eq-ewens-pmf. It accepts a vector with class memberships and
tabulates the number of classes that appear once, twice, and so on.

The `rewens` function samples from the Ewens distribution by following
the Chinese Restaurant Process.

The probability mass function given in @eq-crt-pmf is implemented in
`dewens_k`; the expression for the mean K in @eq-crt-expectation in
function `ewens_k_exact`.

Finally, function `ewens_mle` takes an input vector and estimates $\theta$.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ewens_k_exact.R
\name{ewens_k_exact}
\alias{ewens_k_exact}
\title{Calculate expected number of classes in a sample of size n given theta}
\usage{
ewens_k_exact(n, theta)
}
\arguments{
\item{n}{The sample size}

\item{theta}{The non-negative parameter governing expected sample diversity}
}
\description{
The expected number of classes from the Ewens distribution is given by \eqn{\theta \sum_{j=1}^{n} \frac{1}{\theta + j - 1}}. This is often more convenient than integrating across the PMF given by \code{dewens_k}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dewens.R
\name{dewens}
\alias{dewens}
\title{Probability mass function for the Ewens distribution}
\usage{
dewens(x, theta = 1, log = FALSE)
}
\arguments{
\item{x}{A vector giving class memberships of each observation in the sample}

\item{theta}{A non-negative parameter governing the expected sample diversity.}

\item{log}{if TRUE, probabilities are given as log(p). Default is FALSE.}
}
\value{
A numeric vector giving a probability (or if \code{log = TRUE}, a log probability)
}
\description{
Gives the probability mass function for the Ewens distribution, as described in Ewens, Warren (1972). "The sampling theory of selectively neutral alleles". \emph{Theoretical Population Biology}. 3: 87–112. doi:10.1016/0040-5809(72)90035-4.
}
\details{
The probability of a vector of counts \eqn{m_1, ..., m_n} is given by the expression \deqn{\frac{n!}{\theta (\theta + 1) ... (\theta + n - 1)}\prod_{j=1}^n \frac{\theta^{m_j}}{j^{m_j} m_j!}}
}
\examples{
x <- sample(LETTERS, 120, replace = TRUE)
dewens(x, theta = 1)
dewens(x, theta = 0) ## returns NaN since vector incompatible with zero diversity

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gcrp.R
\name{gcrp}
\alias{gcrp}
\title{Draw from a generalized Chinese Restaurant Process}
\usage{
gcrp(n, alpha = 0, theta = 1)
}
\arguments{
\item{n}{The sample size.}

\item{alpha}{A parameter between zero and one inclusive governing the expected sample diversity}

\item{theta}{A non-negative parameter governing the expected sample diversity.}
}
\value{
A vector of length n consisting of numeric class labels.
}
\description{
Draw from a generalized Chinese Restaurant Process
}
\examples{
rewens(100, 1)
rewens(120, 0.5)
rewens(10, 0)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dewens_k.R
\name{dewens_k}
\alias{dewens_k}
\title{Probability mass function for the number of classes from a Ewens distribution}
\usage{
dewens_k(k, n, theta)
}
\arguments{
\item{k}{An integer number of classes at which to evaluate the PMF}

\item{n}{A sample size not less than k}

\item{theta}{A non-negative parameter governing the expected sample diversity.}
}
\value{
The probability of observing k classes
}
\description{
Probability mass function for the number of classes from a Ewens distribution
}
\details{
The number of classes from a Ewens distribution with parameter \eqn{\theta} is given by the expression \deqn{Pr(K = k) = \lvert{} S^k_n \rvert{} \frac{\theta^k}{\theta (\theta + 1) ... (\theta + n - 1)}}, where \eqn{\lvert{}S^k_n \rvert{}} is the absolute value of a Stirling number of the first kind.
}
\examples{
x <- sample(LETTERS, 120, replace = TRUE)
dewens_k(1, 20, theta = 1) ## Pretty unlikely we just see one class

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rgem.R
\name{rgem}
\alias{rgem}
\title{Draw from the Griffiths-Engen-McCloskey distribution}
\usage{
rgem(alpha = 0, theta = 1, trunc_at = 500)
}
\arguments{
\item{alpha}{A parameter between zero and one}

\item{theta}{A parameter which must be greater than -alpha}

\item{trunc_at}{An integer which specifies the maximum number of components to return}
}
\value{
A vector of shares of length \code{trunc_at} which may sum to less than one
}
\description{
Draw from the Griffiths-Engen-McCloskey distribution
}
\details{
The Griffiths-Engen-McCloskey distribution is the infinite dimensional counterpart to the Ewens sampling distribution. This function does not return an infinite dimensional vector(!), but returns a vector of shares creating by a "stick-breaking" construction. The vector of shares is returned after \code{trunc_at} sticks are broken; this can mean that there is still a non-negligible residual amount.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ewens_mle.R
\name{ewens_mle}
\alias{ewens_mle}
\title{Maximum likelihood estimate of theta given sample vector with class memberships}
\usage{
ewens_mle(x)
}
\arguments{
\item{x}{A vector containing class memberships; sample size n and number of classes k are calculated from this}
}
\value{
A scalar giving the estimate of theta
}
\description{
Maximum likelihood estimate of theta given sample vector with class memberships
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rewens.R
\name{rewens}
\alias{rewens}
\title{Draw from the Ewens distribution}
\usage{
rewens(n, theta = 1)
}
\arguments{
\item{n}{The sample size.}

\item{theta}{A non-negative parameter governing the expected sample diversity.}
}
\value{
A vector of length n consisting of numeric class labels.
}
\description{
Returns a vector with class membership
}
\details{
Although this command is described as sampling from the Ewens distribution, it is easier to think of it as a particular instantiation of the Chinese Restaurant Process, run for n "customers". The $j$th customer
\itemize{
\item sits at a new table with probability \eqn{\frac{\theta}{j - 1 + \theta}}, or
\item sits at an occupied table with probability \eqn{\frac{c}{j - 1 + \theta}}
where $c$ is the number of customers already at each table.
}
}
\examples{
rewens(100, 1)
rewens(120, 0.5)
rewens(10, 0) ## equal to rep(1, 10)
}
